﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MVB_RestBookService
{
    /// <summary>
    /// Transforms an JsonDataContainerDetail and replaces some Fields with a more suitable Value for Frontend Presentation.
    /// </summary>
    public class Transformer
    {
        public static JsonDataContainerDetail transform(JsonDataContainerDetail input)
        {
            //Variables
            JsonDataContainerDetail output = new JsonDataContainerDetail();

            //Copy Root Values
            #region copyRootValues

            output.id = String.Copy(input.id);

            output.publicationDate = String.Copy(input.publicationDate);
            output.availability = String.Copy(input.availability);
            output.publishingStatus = String.Copy(input.publishingStatus);
            output.createdDate = String.Copy(input.createdDate);
            output.lastModifiedDate = String.Copy(input.lastModifiedDate);

            #endregion

            //Create Composites
            #region createCompositesLevel_1

            output.classifications = new Classifications[input.classifications.Length];
            output.collections = new Collections[input.collections.Length];
            output.contributors = new Contributors[input.contributors.Length];
            output.extent = new Extent();
            output.form = new Productform();
            output.identifiers = new Identifiers[input.identifiers.Length];
            output.mediaFiles = new MediaFiles[input.mediaFiles.Length];
            output.prices = new Prices[input.prices.Length];
            output.publishers = new Publishers[input.publishers.Length];
            output.titles = new Titles[input.titles.Length];
            output.texts = new Texts[input.texts.Length];
			output.webSites = new WebSites[input.webSites.Length];
            output.wholesalers = new Wholesalers[input.wholesalers.Length];

            #endregion

            //Copy Composite Values



            //Classifications
            #region classifications

            for (int i = 0; i < output.classifications.Length; i++)
            {
                output.classifications[i] = new Classifications();
                output.classifications[i].id = String.Copy(input.classifications[i].id);
                output.classifications[i].type = String.Copy(input.classifications[i].type);
                output.classifications[i].systematicVersion = String.Copy(input.classifications[i].systematicVersion);
                output.classifications[i].code = String.Copy(input.classifications[i].code);
                output.classifications[i].value = String.Copy(input.classifications[i].value);
                output.classifications[i].schemeName = String.Copy(input.classifications[i].schemeName);
                output.classifications[i].sourceName = String.Copy(input.classifications[i].sourceName);
                output.classifications[i].main = String.Copy(input.classifications[i].main);
            }

            #endregion

            //Collections
            #region collections

            for (int i = 0; i < output.collections.Length; i++)
            {
                output.collections[i] = new Collections();
                output.collections[i].id = String.Copy(input.collections[i].id);

                output.collections[i].master = new Master();
                output.collections[i].master.id = input.collections[i].master.id;
                output.collections[i].master.type = input.collections[i].master.type;

                output.collections[i].master.identifiers = new Identifiers[input.collections[i].master.identifiers.Length];

                for (int j = 0; j < input.collections[i].master.identifiers.Length; j++)
                {
                    output.collections[i].master.identifiers[j] = new Identifiers();
                    output.collections[i].master.identifiers[j].id = input.collections[i].master.identifiers[j].id != null ? String.Copy(input.collections[i].master.identifiers[j].id) : "";
                    output.collections[i].master.identifiers[j].typeName = input.collections[i].master.identifiers[j].typeName != null ? String.Copy(input.collections[i].master.identifiers[j].typeName) : "";
                    output.collections[i].master.identifiers[j].type = input.collections[i].master.identifiers[j].type != null ? String.Copy(input.collections[i].master.identifiers[j].type) : "";
                    output.collections[i].master.identifiers[j].value = input.collections[i].master.identifiers[j].value != null ? String.Copy(input.collections[i].master.identifiers[j].value) : "";
                }

                output.collections[i].master.title = input.collections[i].master.title;
                output.collections[i].master.abbreviatedTitle = input.collections[i].master.abbreviatedTitle;
                output.collections[i].master.subtitle = input.collections[i].master.subtitle;
                output.collections[i].master.publisherId = input.collections[i].master.publisherId;
                output.collections[i].master.publisherNameVlb = input.collections[i].master.publisherNameVlb;
                output.collections[i].master.publisherNameAdb = input.collections[i].master.publisherNameAdb;
                output.collections[i].master.lastModifiedDate = input.collections[i].master.lastModifiedDate;
                output.collections[i].master.active = input.collections[i].master.active;
                output.collections[i].master.bitNo = input.collections[i].master.bitNo;
                output.collections[i].master.aggregatorId = input.collections[i].master.aggregatorId;
                output.collections[i].master.concatenateTitle = input.collections[i].master.concatenateTitle;
                output.collections[i].master.concatenateSubtitle = input.collections[i].master.concatenateSubtitle;

                output.collections[i].sequence = String.Copy(input.collections[i].sequence);
                output.collections[i].levelSequence = String.Copy(input.collections[i].levelSequence);
                output.collections[i].titleInCollection = String.Copy(input.collections[i].titleInCollection);
            }

            #endregion

            //Contributors
            #region contributors

            for (int i = 0; i < output.contributors.Length; i++)
            {
                output.contributors[i] = new Contributors();
                output.contributors[i].id = String.Copy(input.contributors[i].id);
                output.contributors[i].sequence = String.Copy(input.contributors[i].sequence);
                output.contributors[i].type = String.Copy(input.contributors[i].type);
                output.contributors[i].sequenceWithinRole = String.Copy(input.contributors[i].sequenceWithinRole);
                output.contributors[i].firstName = String.Copy(input.contributors[i].firstName);
                output.contributors[i].lastName = String.Copy(input.contributors[i].lastName);
                output.contributors[i].groupName = String.Copy(input.contributors[i].groupName);
                output.contributors[i].dnbId = String.Copy(input.contributors[i].dnbId);
                output.contributors[i].professionalAffiliation = String.Copy(input.contributors[i].professionalAffiliation);
                output.contributors[i].professionalPosition = String.Copy(input.contributors[i].professionalPosition);
                output.contributors[i].biographicalNote = String.Copy(input.contributors[i].biographicalNote);
                output.contributors[i].dateOfBirth = String.Copy(input.contributors[i].dateOfBirth);
                output.contributors[i].dateOfDeath = String.Copy(input.contributors[i].dateOfDeath);
                output.contributors[i].dateOfBirthFormat = String.Copy(input.contributors[i].dateOfBirthFormat);
                output.contributors[i].dateOfDeathFormat = String.Copy(input.contributors[i].dateOfDeathFormat);
                output.contributors[i].language = String.Copy(input.contributors[i].language);

                output.contributors[i].webSites = new WebSites[input.contributors[i].webSites.Length];

                for (int j = 0; j < input.contributors[i].webSites.Length; j++)
				{
					output.contributors[i].webSites[j] = new WebSites();
					output.contributors[i].webSites[j].id = String.Copy(input.contributors[i].webSites[j].id);
					output.contributors[i].webSites[j].type = String.Copy(input.contributors[i].webSites[j].type);
					output.contributors[i].webSites[j].url = String.Copy(input.contributors[i].webSites[j].url);
					output.contributors[i].webSites[j].description = String.Copy(input.contributors[i].webSites[j].description);
					output.contributors[i].webSites[j].productSpecific = String.Copy(input.contributors[i].webSites[j].productSpecific);
				}

                //output.contributors[i].webSites = String.Copy(input.contributors[i].webSites);

                output.contributors[i].company = String.Copy(input.contributors[i].company);
                output.contributors[i].fullName = String.Copy(input.contributors[i].fullName);
                output.contributors[i].value = String.Copy(input.contributors[i].value);
            }

            #endregion

            //Extent
            #region extent

            output.extent.extent = String.Copy(input.extent.extent);
            output.extent.duration = String.Copy(input.extent.duration);
            output.extent.durationUnitCode = String.Copy(input.extent.durationUnitCode);
            output.extent.fileSize = String.Copy(input.extent.fileSize);
            output.extent.fileSizeUnitCode = String.Copy(input.extent.fileSizeUnitCode);
            output.extent.mapScale = String.Copy(input.extent.mapScale);
            output.extent.numIllustrations = String.Copy(input.extent.numIllustrations);
            output.extent.numTracks = String.Copy(input.extent.numTracks);
            output.extent.illustrationText = String.Copy(input.extent.illustrationText);
            output.extent.pages = String.Copy(input.extent.pages);
            output.extent.pagesRoman = String.Copy(input.extent.pagesRoman);
            output.extent.pagesArabic = String.Copy(input.extent.pagesArabic);
            output.extent.fileFormat = String.Copy(input.extent.fileFormat);
            output.extent.epubTypeDescription = String.Copy(input.extent.epubTypeDescription);
            output.extent.drm = String.Copy(input.extent.drm);
            output.extent.drmCode = String.Copy(input.extent.drmCode);
            output.extent.epubTypeVersion = String.Copy(input.extent.epubTypeVersion);

            #endregion

            //Form
            #region form

            output.form.id = String.Copy(input.form.id);
            output.form.type = String.Copy(input.form.type);
            output.form.binding = String.Copy(input.form.binding);
            output.form.description = String.Copy(input.form.description);
            output.form.detail = String.Copy(input.form.detail);
            output.form.diameter = String.Copy(input.form.diameter);
            output.form.height = String.Copy(input.form.height);
            output.form.width = String.Copy(input.form.width);
            output.form.weight = String.Copy(input.form.weight);
            output.form.thickness = String.Copy(input.form.thickness);
            output.form.packaging = String.Copy(input.form.packaging);
            //output.form.productContentType = String.Copy(input.form.productContentType);
            output.form.numberOfPieces = String.Copy(input.form.numberOfPieces);

            #endregion

            //Identifiers
            #region identifiers

            for (int i = 0; i < output.identifiers.Length; i++)
            {
                output.identifiers[i] = new Identifiers();
                output.identifiers[i].id = input.identifiers[i].id != null ? String.Copy(input.identifiers[i].id) : "";
                output.identifiers[i].typeName = input.identifiers[i].typeName != null ? String.Copy(input.identifiers[i].typeName) : "";
                output.identifiers[i].type = input.identifiers[i].type != null ? String.Copy(input.identifiers[i].type) : "";
                output.identifiers[i].value = input.identifiers[i].value != null ? String.Copy(input.identifiers[i].value) : "";
            }

            #endregion

            //MediaFiles
            #region mediaFiles

            for (int i = 0; i < output.mediaFiles.Length; i++)
            {
                output.mediaFiles[i] = new MediaFiles();
                output.mediaFiles[i].id = String.Copy(input.mediaFiles[i].id);
                output.mediaFiles[i].type = String.Copy(input.mediaFiles[i].type);
                output.mediaFiles[i].format = String.Copy(input.mediaFiles[i].format);
                output.mediaFiles[i].linkType = String.Copy(input.mediaFiles[i].linkType);
                output.mediaFiles[i].value = String.Copy(input.mediaFiles[i].value);
                output.mediaFiles[i].fileDate = String.Copy(input.mediaFiles[i].fileDate);
                output.mediaFiles[i].imageResolution = String.Copy(input.mediaFiles[i].imageResolution);
                output.mediaFiles[i].validFrom = String.Copy(input.mediaFiles[i].validFrom);
                output.mediaFiles[i].validUntil = String.Copy(input.mediaFiles[i].validUntil);
                output.mediaFiles[i].assetFileId = String.Copy(input.mediaFiles[i].assetFileId);
                output.mediaFiles[i].externallyGenerated = String.Copy(input.mediaFiles[i].externallyGenerated);
            }

            #endregion

            //Prices
            #region prices

            for (int i = 0; i < output.prices.Length; i++)
            {
                output.prices[i] = new Prices();
                output.prices[i].id = String.Copy(input.prices[i].id);
                output.prices[i].type = String.Copy(input.prices[i].type);
                output.prices[i].country = String.Copy(input.prices[i].country);
                output.prices[i].territory = String.Copy(input.prices[i].territory);
                output.prices[i].currency = String.Copy(input.prices[i].currency);
                output.prices[i].validFrom = String.Copy(input.prices[i].validFrom);
                output.prices[i].validUntil = String.Copy(input.prices[i].validUntil);
                output.prices[i].state = String.Copy(input.prices[i].state);
                output.prices[i].taxRate = String.Copy(input.prices[i].taxRate);
                output.prices[i].value = String.Copy(input.prices[i].value);
                output.prices[i].typeQualifier = String.Copy(input.prices[i].typeQualifier);
                output.prices[i].typeDescription = String.Copy(input.prices[i].typeDescription);
                output.prices[i].discountCodeText = String.Copy(input.prices[i].discountCodeText);
                output.prices[i].minimumQuantity = String.Copy(input.prices[i].minimumQuantity);
                output.prices[i].priceTypeGroup = String.Copy(input.prices[i].priceTypeGroup);
                output.prices[i].fixedRetailPrice = String.Copy(input.prices[i].fixedRetailPrice);
                output.prices[i].provisional = String.Copy(input.prices[i].provisional);
            }

            #endregion

            //Publishers
            #region publishers

            for (int i = 0; i < output.publishers.Length; i++)
            {
                output.publishers[i] = new Publishers();
                output.publishers[i].id = String.Copy(input.publishers[i].id);
                output.publishers[i].type = String.Copy(input.publishers[i].type);
                output.publishers[i].name = String.Copy(input.publishers[i].name);
                output.publishers[i].adbName = String.Copy(input.publishers[i].adbName);
                output.publishers[i].nameCodeType = String.Copy(input.publishers[i].nameCodeType);
                output.publishers[i].publisherId = String.Copy(input.publishers[i].publisherId);
                output.publishers[i].imprint = String.Copy(input.publishers[i].imprint);

                output.publishers[i].webSites = new WebSites[input.publishers[i].webSites.Length];

                for (int j = 0; j < input.publishers[i].webSites.Length; j++)
				{
					output.publishers[i].webSites[j] = new WebSites();
					output.publishers[i].webSites[j].id = String.Copy(input.publishers[i].webSites[j].id);
					output.publishers[i].webSites[j].type = String.Copy(input.publishers[i].webSites[j].type);
					output.publishers[i].webSites[j].url = String.Copy(input.publishers[i].webSites[j].url);
					output.publishers[i].webSites[j].description = String.Copy(input.publishers[i].webSites[j].description);
					output.publishers[i].webSites[j].productSpecific = String.Copy(input.publishers[i].webSites[j].productSpecific);
				}
            }
            
            #endregion

            //Texts
            #region texts

            for (int i = 0; i < output.texts.Length; i++)
            {
                output.texts[i] = new Texts();
                output.texts[i].id = String.Copy(input.texts[i].id);
                output.texts[i].type = String.Copy(input.texts[i].type);
                output.texts[i].formatCode = String.Copy(input.texts[i].formatCode);
                output.texts[i].language = String.Copy(input.texts[i].language);
                output.texts[i].value = String.Copy(input.texts[i].value);
                output.texts[i].linkType = String.Copy(input.texts[i].linkType);
                output.texts[i].link = String.Copy(input.texts[i].link);
                output.texts[i].validFrom = String.Copy(input.texts[i].validFrom);
                output.texts[i].validUntil = String.Copy(input.texts[i].validUntil);
                output.texts[i].textPublicationDate = String.Copy(input.texts[i].textPublicationDate);
            }

            #endregion

            //Titles
            #region titles

            for (int i = 0; i < output.titles.Length; i++)
            {
                output.titles[i] = new Titles();
                output.titles[i].id = String.Copy(input.titles[i].id);
                output.titles[i].type = String.Copy(input.titles[i].type);
                output.titles[i].subtitle = String.Copy(input.titles[i].subtitle);
                output.titles[i].title = String.Copy(input.titles[i].title);
            }

            #endregion

			//WebSites
			#region webSites

			for (int i = 0; i < output.webSites.Length; i++)
			{
				output.webSites[i] = new WebSites();
				output.webSites[i].id = String.Copy(input.webSites[i].id);
				output.webSites[i].type = String.Copy(input.webSites[i].type);
				output.webSites[i].url = String.Copy(input.webSites[i].url);
				output.webSites[i].description = String.Copy(input.webSites[i].description);
				output.webSites[i].productSpecific = String.Copy(input.webSites[i].productSpecific);
			}

            #endregion

            //Wholesaler
            #region wholesalers

            for (int i = 0; i < output.wholesalers.Length; i++)
            {
                output.wholesalers[i] = new Wholesalers();
                output.wholesalers[i].id = String.Copy(input.wholesalers[i].id);
                output.wholesalers[i].isbn = String.Copy(input.wholesalers[i].isbn);
                output.wholesalers[i].sigl = String.Copy(input.wholesalers[i].sigl);
                output.wholesalers[i].orderNumber = String.Copy(input.wholesalers[i].orderNumber);
                output.wholesalers[i].recordSource = String.Copy(input.wholesalers[i].recordSource);
            }

            #endregion



            //Start Transforming

            //Prices -> provisional
            #region prices_provisional

            for (int i = 0; i < output.prices.Length; i++)
            {
                switch (output.prices[i].provisional)
                {
                    case "True":
                        output.prices[i].provisional = "ca.";
                        break;
                    case "False":
                        output.prices[i].provisional = "";
                        break;
                    default:
                        output.prices[i].provisional = "";
                        break;
                }
            }

            #endregion

            //Prices -> taxRate
            #region prices_taxRate

            for (int i = 0; i < output.prices.Length; i++)
            {
                switch (output.prices[i].taxRate)
                {
                    case "S":
                        output.prices[i].taxRate = "Normaler Satz";
                        break;
                    case "R":
                        output.prices[i].taxRate = "Reduzierter Satz";
                        break;
                    case "Z":
                        output.prices[i].taxRate = "Keine Steuer";
                        break;
                    default:
                        output.prices[i].taxRate = "";
                        break;
                }
            }

            #endregion

            //Prices -> fixedRetailPrice
            #region prices_fixedRetailPrice

            for (int i = 0; i < output.prices.Length; i++)
            {
                switch (output.prices[i].fixedRetailPrice)
                {
                    case "True":
                        output.prices[i].fixedRetailPrice = "Gebundener Ladenpreis";
                        break;
                    case "False":
                        output.prices[i].fixedRetailPrice = "UVP";
                        break;
                    default:
                        output.prices[i].fixedRetailPrice = "";
                        break;
                }
            }

            #endregion

            //Prices -> value
            #region prices_value

            for (int i = 0; i < output.prices.Length; i++)
            {
                output.prices[i].value = double.Parse(output.prices[i].value.Replace(".", ",")).ToString("N2");
            }

            #endregion

            //

            return output;
        }
    }
}
